import 'dart:io';
import 'dart:ffi';
import 'dart:isolate';
import 'package:ffi/ffi.dart';
import 'package:path/path.dart';

import 'package:libwinmedia/generated/bindings.dart';

void main(List<String> args) {
  var path =
      (Platform.script.path.split('/')..removeLast()).join('/').substring(1) +
          '/../libwinmedia.dll';
  var lwm = LWM(DynamicLibrary.open(path));
  if (args.isEmpty) {
    print('No URI provided.');
    print('Example Usage:');
    print(
        'dart ${basename(Platform.script.path)} file://C:/alexmercerind/music.mp3');
    print(
        'dart ${basename(Platform.script.path)} https://alexmercerind.github.io/video.mp4');
    return;
  }
  var receiver = ReceivePort()
    ..listen(
      (message) {
        print(message);
      },
    );
  lwm.InitializeDartApi(NativeApi.postCObject, receiver.sendPort.nativePort);
  lwm.PlayerCreate(0, false, ''.toNativeUtf16());
  lwm.PlayerSetRateEventHandler(0, Pointer.fromFunction(doubleC));
  lwm.PlayerSetVolumeEventHandler(0, Pointer.fromFunction(doubleC));
  lwm.PlayerSetPositionEventHandler(0, Pointer.fromFunction(intC));
  lwm.PlayerSetDurationEventHandler(0, Pointer.fromFunction(intC));
  lwm.PlayerSetIndexEventHandler(0, Pointer.fromFunction(intC));
  lwm.PlayerSetIsPlayingEventHandler(0, Pointer.fromFunction(intC));
  lwm.PlayerSetIsBufferingEventHandler(0, Pointer.fromFunction(intC));
  lwm.PlayerSetIsCompletedEventHandler(0, Pointer.fromFunction(intC));
  lwm.PlayerSetVolume(0, 0.5);
  lwm.PlayerSetRate(0, 1.2);
  lwm.PlayerOpen(
    0,
    args.length,
    args.toNativeUtf16Array(),
    List.generate(args.length, (index) => index).toNativeArray(),
  );
  lwm.PlayerPlay(0);
}

void intC(int _) {}
void doubleC(double _) {}

extension NativeTypesString on List<String> {
  Pointer<Pointer<Utf16>> toNativeUtf16Array() {
    final list =
        map((string) => string.toNativeUtf16()).toList().cast<Pointer<Utf16>>();
    final pointer = calloc.allocate<Pointer<Utf16>>(list.length * 8);
    for (var index = 0; index < length; index++) {
      pointer[index] = list[index];
    }
    return pointer;
  }
}

extension NativeTypesInt on List<int> {
  Pointer<Int32> toNativeArray() {
    final pointer = calloc.allocate<Int32>(length * 4);
    for (var index = 0; index < length; index++) {
      pointer[index] = this[index];
    }
    return pointer;
  }
}
